#!/usr/bin/env python

# Rss Notify
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer


import os
from os.path import join, dirname, pardir, abspath
import gtk

import sys


VERSION = "0.6"
USR_PATH = None

c = abspath(dirname(__file__))
while True:
    ls = os.listdir(c)
    if "bin" in ls and "share" in ls:
        USR_PATH = c
        break
    
    c = join(c, pardir)


IMAGE_PATH = join(USR_PATH, 'share', 'rss-notify', 'images')
GLADE_PATH = join(USR_PATH, 'share', 'rss-notify', 'glade')
ICON_PATH  = join(USR_PATH, 'share', 'icons')

_autostart_dir = join(os.getenv("HOME"), ".config", "autostart")
_desktop_file  = join(USR_PATH, 'share', 'applications',
        'rss-notify.desktop')

ICON_NAME = 'rss-notify'
icon_theme = gtk.icon_theme_get_default()

def get_icon_pixbuf(size):
    global icon_theme
    global ICON_NAME
    
    iconInfo = icon_theme.lookup_icon(ICON_NAME, size,
        gtk.ICON_LOOKUP_NO_SVG)
    
    if iconInfo:
        # icon is found
        base_size = iconInfo.get_base_size()
        if base_size != size:
            ## No sizexsize icon in the users theme so use the default
            icon_theme = gtk.IconTheme()
            icon_theme.set_search_path((ICON_PATH,))
    else:
        icon_theme.append_search_path(ICON_PATH)

    pixbuf = icon_theme.load_icon(ICON_NAME, size,
                gtk.ICON_LOOKUP_NO_SVG)

    return pixbuf

## gconf stuff
from rss_notify.config import Configurator

_key = '/apps/rss-notify/prefs/'

_conf = Configurator()

### register all the options that will be used.
_conf.register_opt("update_interval", os.path.join(_key,
    "update_interval"), 5)

_conf.register_opt("num_of_items", os.path.join(_key,
    "num_of_items"), 3)

_conf.register_opt("notification_icon", os.path.join(_key,
    "notification_icon"), 0)

_conf.register_opt("autostart", os.path.join(_key,
    "autostart"), True)


## Add Rss Notify to startup programs
import shutil

def add_to_startup():
    """Adds rss-notify to the programs that start
    on login.
    """
    if not os.path.exists(_autostart_dir):
        os.makedirs(_autostart_dir)
    shutil.copy(_desktop_file, _autostart_dir)

def remove_from_startup():
    """Removes rss-notify from the programs that start
    on login.
    """

    filename = os.path.join(_autostart_dir, "rss-notify.desktop")
    if os.path.exists(filename):
        os.remove(filename)

if _conf.get("autostart").get_bool():
    add_to_startup()
else:
    remove_from_startup()

def get_configurator():
    return _conf

from rss_notify import feedmanager
## Feed management
if os.getenv("XDG_DATA_HOME"):
    data_dir = os.getenv("XDG_DATA_HOME")
else:
    data_dir = os.path.join(os.getenv("HOME"), ".local", "share")


data_dir = os.path.join(data_dir, "rss-notify")

DATA_DIR = data_dir

if not os.path.exists(data_dir):
    os.makedirs(data_dir)

opml_file = os.path.join(data_dir, "feedlist.opml")

_FM = feedmanager.FeedManager(opml_file)

def get_feedmanager():
    return _FM

### Setup translations
###
GETTEXT_DOMAIN = "rss-notify"
LOCALE_PATH = os.path.join(USR_PATH, "share", "locale")

import gtk.glade
import gettext
import locale

locale.setlocale(locale.LC_ALL, '')

for module in gtk.glade, gettext:
    module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
    module.textdomain(GETTEXT_DOMAIN)

# register the gettext function for the whole interpreter as "_"
import __builtin__
__builtin__._ = gettext.gettext


from rss_notify.main import main
