#!/usr/bin/env python

# Rss Notify
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer

import gtk
import gobject

import os
import sys
import webbrowser
import threading
import time

# import local modules here
import rss_finder
import rss_notify

builder = gtk.Builder()
glade_file = os.path.join(rss_notify.GLADE_PATH, "gui.glade")
try:
    builder.add_from_file(glade_file)

except Exception, data:
    print data
    print _("Error loading glade file")
    sys.exit(1)

### A Dictionary to store all the ElementTree elements
KEY = 0
ELEMENTS = {}

def add_element(element):
    global KEY
    global ELEMENTS

    KEY += 1
    ELEMENTS[KEY] = element
    return KEY

get = builder.get_object

class Window(object):

    def __init__(self):
        
        self.window = get("window")

        self.import_last_dir = None
        self.export_last_dir = None


        # treeview stuff
        treeview = get("treeview")
        self.treeview = treeview
        self.selection = treeview.get_selection()
        self.selection.set_mode(gtk.SELECTION_MULTIPLE)

        treestore = get("treestore")
        ## sort the treestore
        treestore.set_sort_func(10, self.sort_treestore)
        treestore.set_sort_column_id(10, gtk.SORT_ASCENDING)

        self.treestore = treestore
        
        ## retrieve the object for managing feeds
        self.FM = rss_notify.get_feedmanager()

        self.FM.set_treeview(self.treeview)
        self.FM.update_treeview()
        
        self.feed_menu = get("feed_menu")
        self.feed_menuitem_browser = get("feed_menuitem_browser")

        ## connect signals
        builder.connect_signals(self)
    
    def on_treeview_button_press_event(self, treeview, event, data=None):

        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            pInfo = treeview.get_path_at_pos(x, y)
            if pInfo != None:
                path, col, cellx, celly = pInfo
                model = treeview.get_model()


                if model[path][3] == 1:
                    self.feed_menuitem_browser.set_sensitive(False)
                else:
                    self.feed_menuitem_browser.set_sensitive(True)
                    

                treeview.grab_focus()

                sel_paths = self.selection.get_selected_rows()[1]

                if path not in sel_paths:
                    self.selection.unselect_all()
                    self.selection.select_path(path)
                    sel_paths = self.selection.get_selected_rows()[1]

                if sel_paths:
                    for p in sel_paths:
                        self.selection.select_path(p)
                else:
                    treeview.set_cursor(path, col, 0)
                    
                self.feed_menu.popup(None, None, None,
                        event.button, event.time)

                ## must return true or else only one row will be selected
                ## see http://www.daa.com.au/pipermail/pygtk/2005-June/010465.html
                return True



    def sort_treestore(self, model, iter1, iter2, data=None):
        v31 = model.get_value(iter1, 3)
        v32 = model.get_value(iter2, 3)

        if v31 > v32:
            return -1
        elif v31 < v32:
            return 1


        v11 = model.get_value(iter1, 1)
        v12 = model.get_value(iter2, 1)
        if v11:
            v11 = v11.upper()
        if v12:
            v12 = v12.upper()

        if v11 < v12:
            return -1
        elif v11 > v12:
            return 1

        return 0

    # Callbacks

    def on_window_delete_event(self, window, event, data=None):

        self.window.hide_on_delete()
        return True

    def on_treeviewcolumn_clicked(self, tvc, data=None):
        sort_order = tvc.get_sort_order()
        if sort_order == gtk.SORT_ASCENDING:
            sort_order = tvc.set_sort_order(gtk.SORT_DESCENDING)
            self.treestore.set_sort_column_id(10, gtk.SORT_DESCENDING)
        else:
            sort_order = tvc.set_sort_order(gtk.SORT_ASCENDING)
            self.treestore.set_sort_column_id(10, gtk.SORT_ASCENDING)
    
    def on_feed_menuitem_browser_activate(self, menuitem, data=None):
        model, paths = self.selection.get_selected_rows()
        for path in paths:
            element = self.FM.get_feed(model[path][2])
            url = element.get("htmlUrl")
            webbrowser.open(url)


        
    
    ## This function does not actually add the feeds,
    ## that is done in AddFeedDialog.show_feeds, this removes
    ## any feeds that the user unchecked in the AddFeedDialog.

    def on_AddFeed_activate(self, action, data=None):

        print 1
        #self.populate_treestore()

        dialog = AddFeedDialog()
        dialog.dialog.set_transient_for(self.window)

        dialog.run()
        print 2
            
        gobject.idle_add(self.FM.update_treeview)
        gobject.idle_add(self.FM.save_feeds)

            
    def on_RemoveFeed_activate(self, action, data=None):
       
        model, paths = self.selection.get_selected_rows()
        paths.reverse()
        for path in paths:
            iter = model.get_iter(path)
            key = model.get_value(iter, 2)
            type = model.get_value(iter, 3)
            if type == 0:
                element = self.FM.get_feed(key) #ELEMENTS[model.get_value(iter, 2)]
            else:
                element = self.FM.get_category(key) #ELEMENTS[model.get_value(iter, 2)]

            self.FM.remove_feed(element)
            model.remove(iter)

        self.FM.save_feeds()


    
    def filechooser(self, title, action):
        if action == gtk.FILE_CHOOSER_ACTION_OPEN:
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_OPEN, gtk.RESPONSE_OK)
        else:
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_SAVE, gtk.RESPONSE_OK)
        
        dialog = gtk.FileChooserDialog(title, self.window,
                action, buttons)

        return dialog

    def on_import_menu_item_activate(self, menuitem, data=None):
        dialog = self.filechooser(_("Import Feed List"),
                gtk.FILE_CHOOSER_ACTION_OPEN)

        dialog.set_transient_for(self.window)
        dialog.set_do_overwrite_confirmation(True)

        filter = gtk.FileFilter()
        filter.set_name(_("All files"))
        filter.add_pattern("*")
        dialog.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("OPML Files")
        filter.add_mime_type("text/x-opml+xml")
        filter.add_pattern("*.opml")
        filter.add_mime_type("application/xml")
        filter.add_pattern("*.xml")

        dialog.add_filter(filter)
        dialog.set_filter(filter)

        if self.import_last_dir:

            dialog.set_current_folder(self.import_last_dir)
        else:

            dialog.set_current_folder(os.getenv("HOME"))

        response = dialog.run()

        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            try:
                self.FM.import_feeds(filename)
                self.FM.update_treeview()

            except Exception, data:
                import traceback
                print traceback.print_exc()
                err_dialog = gtk.MessageDialog(parent=self.window,
                    flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                    type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                    message_format=_("Error: Invalid Feed File"))

                response = err_dialog.run()
                self.import_last_dir = dialog.get_current_folder()
                dialog.destroy()
                err_dialog.destroy()
                return

        self.import_last_dir = dialog.get_current_folder()

        dialog.destroy()

    def on_export_menu_item_activate(self, menuitem, data=None):

        dialog = self.filechooser(_("Export Feed List"),
                gtk.FILE_CHOOSER_ACTION_SAVE)
        dialog.set_transient_for(self.window)

        if self.export_last_dir:
            dialog.set_current_folder(self.export_last_dir)
        else:
            dialog.set_current_folder(os.getenv("HOME"))

        dialog.set_current_name("feedlist.opml")

        response = dialog.run()
        
        if response == gtk.RESPONSE_OK:
            filename = dialog.get_filename()
            try:
                self.FM.export_feeds(filename)
            except Exception, data:
                import traceback
                print traceback.print_exc()
                print data

        self.export_last_dir = dialog.get_current_folder()

        dialog.destroy()


    def on_Preferences_activate(self, action, data=None):
        prefs = Preferences()
        prefs.window.set_transient_for(self.window)
        prefs.show_all()

    def on_About_activate(self, action, data=None):
        dialog = AboutDialog()
        dialog.dialog.set_transient_for(self.window)
        dialog.run()
        dialog.dialog.destroy()

    def on_Quit_activate(self, action, data=None):
        gtk.main_quit()
        sys.exit(0)
    
    def on_close_button_clicked(self, button, data=None):
        self.window.hide()
    
    def toggle_visible(self):

        if self.window.get_property("visible"):
            self.window.hide()
        else:
            self.window.show_all()
   
class AddFeedDialog(object):

    def __init__(self):

        builder = gtk.Builder()
        glade_file = os.path.join(rss_notify.GLADE_PATH, "misc.glade")
        try:
            builder.add_from_file(glade_file)

        except Exception, data:
            print data
            print _("Error loading glade file")
            sys.exit(1)

        get = builder.get_object

        self.dialog = get("add_dialog")
        self.entry = get("add_dialog_entry")
        
        self.feedlist_sw = get("feedlist_sw")

        self.treeview = get("feed_treeview")
        self.feed_liststore = get("feed_liststore")

        self.progressbar = get("progressbar")
        self.progressbar.set_fraction(0.0)



        self.ok_button = get("add_dialog_ok_button")
        self.cancel_button = get("add_dialog_cancel_button")

        self.load_feed_button = get("load_feed_button")

        image = gtk.image_new_from_stock(gtk.STOCK_OK,
                    gtk.ICON_SIZE_BUTTON)

        self.load_feed_button.set_image(image)


        self.cbb = get("add_dialog_combobox")

        self.cat_dialog = get("cat_dialog")
        self.cat_dialog_entry = get("cat_dialog_entry")
        self.cat_dialog_cbb = get("cat_dialog_combobox")
        self.cat_dialog_ok_button = get("cat_dialog_ok_button")


        self.cat_liststore = get("cat_liststore")
        
        self.FM = rss_notify.get_feedmanager()

        print 3
        self.FM.populate_cbb_liststore(self.cat_liststore)
        print 4


        ## add a "new" category

        self.dir_pixbuf = self.dialog.render_icon(gtk.STOCK_DIRECTORY,
                        gtk.ICON_SIZE_MENU)
        self.new_pixbuf = self.dialog.render_icon(gtk.STOCK_NEW,
                        gtk.ICON_SIZE_MENU)

        self.cat_liststore.prepend([None, _("None"), 0])
        self.cat_liststore.append([self.new_pixbuf, _("New..."), 0])
        
        self.cbb.set_active(0)
        self.cat_dialog_cbb.set_active(0)

        builder.connect_signals(self)
    

        self.id = None

        self.thread = None

        self.feeds_to_add = []

        self.loading = False
        self.done = False
        self.destroyed = False

    def _pulse(self):

        self.progressbar.pulse()
        return True
    
    def progressbar_pulse_start(self):

        self.progressbar_pulse_stop()

        self.id = gobject.timeout_add(100, self._pulse)

    def progressbar_pulse_stop(self):

        gobject.idle_add(self._idle_func, self.progressbar.set_fraction, 0.0)
        if self.id != None:
            gobject.source_remove(self.id)

    def on_cell_toggled(self, cellrenderer, row, data=None):
        
        self.feed_liststore[row][0] = not cellrenderer.get_active()

    
    ### Function for calling other functions, used with gobject.idle_add
    ### This is needed because some gtk widget functions return True which 
    ### would cause them to be called indefinetly.

    def _idle_func(self, func, *args):

        func(*args)
        return False

        
    ### Populate feed_liststore with the feeds found 
    ### by scraping the user supplied URL.

    def show_feeds(self, url):
        
        self.feeds_to_add = []

        child = self.feedlist_sw.get_child()

        if child:
            gobject.idle_add(self._idle_func, self.feedlist_sw.remove, child)

        gobject.idle_add(self._idle_func, self.feedlist_sw.add, self.treeview)
        gobject.idle_add(self._idle_func, self.feedlist_sw.show_all)
        ### Show the user that something is going on.
        self.progressbar_pulse_start()

        gobject.idle_add(self._idle_func, self.feed_liststore.clear)
        
        feeds = rss_finder.get_feeds(url)

        ### Either no feeds were found for the url
        ### or the user typed in an invalid url.
        if len(feeds) == 0:
            ### Take out the treeview and
            ### add a label telling  the user
            ### that no feeds were found.
            child = self.feedlist_sw.get_child()
            if child:

                gobject.idle_add(self._idle_func, self.feedlist_sw.remove, child)
            label = gtk.Label(_("No Feeds Found."))
            gobject.idle_add(self._idle_func,
                self.feedlist_sw.add_with_viewport, label)
            ### remove extra shadow.
            def remove_shadow():

                vp = self.feedlist_sw.get_child()
                vp.set_shadow_type(gtk.SHADOW_NONE)
                
            
            gobject.idle_add(remove_shadow)

            gobject.idle_add(self._idle_func, self.feedlist_sw.show_all)

            gobject.idle_add(self._idle_func, self.ok_button.set_sensitive, False)

        for feed_url in feeds:
            
            sens = bool(len(self.feeds_to_add))
            gobject.idle_add(self._idle_func, self.ok_button.set_sensitive, sens)

            elem = self.FM.create_feed(feed_url)
            if elem != False:
                feed_title = elem.get("title")
                self.feeds_to_add.append(elem)

                if self.done:
                    break
                s = ("<span size='medium'><b>"+feed_title+"</b></span>"+"\n"+
                        str(elem.get("htmlUrl", "")))
                gobject.idle_add(self._idle_func,
                    self.feed_liststore.append,
                    [True, s, feed_url])


        ### If the user closed the Dialog all of its widgets will
        ### be destroyed.
        if not self.destroyed:

            if len(feeds):
                gobject.idle_add(self._idle_func, self.ok_button.set_sensitive, True)

            self.set_loading(False)

            self.progressbar_pulse_stop()

    def get_loading(self):
        return self.loading

    def set_loading(self, loading):
        
        self.loading = loading


        sens = not loading

        for widget in self.dialog.get_children()[0]:
            if widget.get_name() != "add_dialog_hbox":
                gobject.idle_add(self._idle_func, widget.set_sensitive, sens)

        gobject.idle_add(self._idle_func, self.entry.set_sensitive, sens)

        ### Change the icon.
        def change_icon(self):

            #self.load_feed_button.remove(child)
            stock_id = [gtk.STOCK_OK, gtk.STOCK_STOP][loading]

            image = gtk.image_new_from_stock(stock_id,
                    gtk.ICON_SIZE_BUTTON)

            self.load_feed_button.set_image(image)

        gobject.idle_add(self._idle_func, change_icon, self)

    def on_load_feed_button_clicked(self, button, data=None):

        if self.get_loading() == False:

            url = self.entry.get_text()
            url = url.strip()


            if url:
                self.set_loading(True)

                self.done = False
                self.thread = threading.Thread(target=self.show_feeds, args=[url])
                self.thread.start()
        else:
            lock = threading.Lock()
            with lock:
                self.done = True
                self.set_loading(False)
                self.progressbar_pulse_stop()



    def on_cat_dialog_entry_changed(self, entry, data=None):
        
        active = bool(self.cat_dialog_entry.get_text().strip())
        self.cat_dialog_ok_button.set_sensitive(active)
        
    def on_add_dialog_entry_changed(self, entry, data=None):

        active = bool(self.entry.get_text().strip())
        self.load_feed_button.set_sensitive(active)

    def on_add_dialog_combobox_changed(self, combobox, data=None):

        model = combobox.get_model()

        old_active = combobox.get_active()

        iter = combobox.get_active_iter()
        active = combobox.get_active()

        sel_text = None
        if iter:
            sel_text = model.get_value(
                    combobox.get_active_iter(), 1)
        
        ### open "new category" dialog.
        if active != -1 and model[active].path == model[-1].path:
            
            element = None
            ### Remove the "New..." item.
            model.remove(model[-1].iter)

            response = self.cat_dialog.run()
            self.cat_dialog.hide()
            
            if response == gtk.RESPONSE_OK:

                name = self.cat_dialog_entry.get_text()
                name = name.strip()
                if name:

                    parent_key = self.cat_liststore.get_value(
                            self.cat_dialog_cbb.get_active_iter(), 2)

                    if parent_key:
                        parent = self.FM.get_category(parent_key)
                    else:
                        parent = None

                    element = self.FM.add_category(name, parent)
                    
                    self.FM.populate_cbb_liststore(model)

            ### Re-add the "New..." item.
            model.append([self.new_pixbuf, _("New..."), 0])

            ### Set the combobox to be on the newly created 
            ### category or None if we didn't create one.
            if element != None:

                iterator = zip(model, range(len(model)))
                for row, n in iterator:
                    if row[2] != 0:
                        if self.FM.get_category(row[2]) is element:
                            self.cbb.set_active(n)
            else:
                self.cbb.set_active(0)

        else:
            self.cbb.set_active(old_active)


    def run(self):
        
        #self.feed_liststore.clear()
        response = self.dialog.run()

        self.entry.set_text("")
        text = ""


        if response == gtk.RESPONSE_OK:

            key = self.cat_liststore.get_value(self.cbb.get_active_iter(),
                    2)

            category = None
            if key != 0:
                category = self.FM.get_category(key)

            ### The feed elements created in show_feeds are not
            ### yet added to the ElementTree.
            lock = threading.Lock()
            with lock:
                iterator = zip(self.feeds_to_add, self.feed_liststore)
                for elem, row in iterator:
                    ### Don't add feeds that were unchecked.
                    if row[0]:
                        self.FM.move_feed(elem, category)

    
        lock = threading.Lock()
        with lock:
            self.done = True
            self.destroyed = True
        
        self.dialog.destroy()


class AboutDialog(object):
    
    def __init__(self):

        builder = gtk.Builder()
        glade_file = os.path.join(rss_notify.GLADE_PATH, "misc.glade")
        try:
            builder.add_from_file(glade_file)

        except Exception, data:
            print data
            print _("Error loading glade file")
            sys.exit(1)

        get = builder.get_object

        self.dialog = get("about_dialog")

    def run(self):
        response = self.dialog.run()
        return response
        
                

class Preferences(object):

    def __init__(self):

        ## update gui from configuration
        self.conf = rss_notify.get_configurator()
        builder = gtk.Builder()

        glade_file = os.path.join(rss_notify.GLADE_PATH, "prefs.glade")
        try:
            builder.add_from_file(glade_file)

        except Exception, data:
            print data
            print _("Error loading glade file")
            sys.exit(1)

        get = builder.get_object

        self.window = get("window")

        rb1 = get("rb1")
        rb2 = get("rb2")

        i = self.conf.get("notification_icon").get_int()
        if i == 0:
            rb1.set_active(True)
        elif i == 1:
            rb2.set_active(True)

        auto_cb = get("autostart_cb")
        auto_cb.set_active(self.conf.get("autostart").get_bool())

        adj1 = get("adjustment1")
        adj1.set_value(self.conf.get("update_interval").get_int())

        adj2 = get("adjustment2")
        adj2.set_value(self.conf.get("num_of_items").get_int())

        builder.connect_signals(self)

    def on_rb1_toggled(self, rb, data=None):
        if rb.get_active():
            self.conf.set("notification_icon", 0)
    
    def on_rb2_toggled(self, rb, data=None):
        if rb.get_active():
            self.conf.set("notification_icon", 1)
    

    def on_autostart_cb_toggled(self, cb, data=None):
        self.conf.set("autostart", cb.get_active())
        if cb.get_active():
            rss_notify.add_to_startup()
        else:
            rss_notify.remove_from_startup()

    def on_adjustment1_value_changed(self, adj, data=None):
        self.conf.set("update_interval", int(adj.get_value()))

    def on_adjustment2_value_changed(self, adj, data=None):
        self.conf.set("num_of_items", int(adj.get_value()))

    def on_close_button_clicked(self, button, data=None):

        self.window.destroy()

    def show_all(self):
        self.window.show_all()
     
_menu = get("status_icon_menu")
def get_status_icon_menu():
    return _menu
