#!/usr/bin/env python

# Rss Notify
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer

try:
    import feedparser
except ImportError:
    print "Error: python-feedparser not installed"
    sys.exit(1)

import pynotify
import gtk
import gobject
import gconf
import threading, thread

import sys
import os
import time
import urllib
import webbrowser

import rss_notify
import config
import feedmanager
import gui


class NotifyThread(threading.Thread):
    
    def __init__(self, debug):

        threading.Thread.__init__(self, target=self.run)
        
        self.debug = debug

        self.FM = rss_notify.get_feedmanager()
        self.FM.connect("feed-list-changed", self.on_feed_list_changed)

        self.conf = rss_notify.get_configurator()
        #
        self.latest = {}
        #
        
        self._stopped = False

        self.capabilities = {'actions':             False,
                'body':                False,
                'body-hyperlinks':     False,
                'body-images':         False,
                'body-markup':         False,
                'icon-multi':          False,
                'icon-static':         False,
                'sound':               False,
                'image/svg+xml':       False,
                'append':              False}

        ## initialize pynotify
        if not pynotify.init("icon-summary-body"):
            print "Can't Initialize pynotify"
            sys.exit(1)

        caps = pynotify.get_server_caps ()
        if caps == None:
            #print "Failed to receive server caps."
            sys.exit (1)

        for cap in caps:
            self.capabilities[cap] = True
        
        #print self.capabilities['image/svg+xml']
        
        i = self.conf.get("notification_icon").get_int()
        
        self.icon_filenames = []

        self.icon_filenames.append(os.path.join(rss_notify.IMAGE_PATH,
                "default.svg"))
        self.icon_filenames.append(os.path.join(rss_notify.IMAGE_PATH,
                "standard.svg"))

    
    def _start(self):
        self._stopped = False

    def _stop(self):
        
        self._stopped = True

    def quit(self):
        thread.exit()

    def sleep(self):
        lock = threading.Lock()
        with lock:
            update_interval = self.conf.get("update_interval").get_int()
        
        for x in range(0, update_interval * 2):
            time.sleep(30)

    
    def on_feed_list_changed(self, source):
        print "on feed list changed"
        self._stop()

    def run(self):
        
        while True:
            ### Download feed info right away
            if not self.debug:
                self.sleep()
        
            lock = threading.Lock()
            with lock:
                feeds = self.FM.get_feed_urls()
                 
            for feed in feeds:
                ## the restart loop
                if self._stopped:
                    self._start()
                    break

                lock = threading.Lock()
                with lock:
                    num_of_items = self.conf.get("num_of_items").get_int()
             

                try:
                    ## download the feed data
                    d = feedparser.parse(feed)

                except Exception, data:
                    print data
                    continue
            
                
                if len(d.entries) < 1:
                    continue

                entries = [e.title for e in d.entries]
                try:

                    l = d.entries[0].title
                    entries = entries[:num_of_items]

                except Exception, data:
                    print data

                if feed in self.latest:
                    if l == self.latest[feed]:
                        continue
                else:
                    if not self.debug:
                        ## Only show items that are new, no longer recap
                        self.latest[feed] = l
                        continue

                self.latest[feed] = l
                summary = l

                i = self.conf.get("notification_icon").get_int()
                filename = self.icon_filenames[i]
                
                pixbuf = gtk.gdk.pixbuf_new_from_file(filename)

                note = pynotify.Notification(d.feed.title,
                        summary, None)

                note.set_icon_from_pixbuf(pixbuf)

                note.set_hint_string("append", "allow")
                note.show()

                time.sleep(2)
                
                # The first item is shown longer
                for e in entries:
                    note.update(d.feed.title, e, filename)
                    
                    note.show()
                    time.sleep(3)
                
                ## Pause in between feeds
                time.sleep(5)
                
            if self.debug:
                self.sleep()

class RssNotify(gtk.StatusIcon):

    def __init__(self, debug):

        gtk.StatusIcon.__init__(self)
        

        ## status icon stuff
        self.set_from_pixbuf(rss_notify.get_icon_pixbuf(22))

        gtk.window_set_default_icon_list(rss_notify.get_icon_pixbuf(16),
            rss_notify.get_icon_pixbuf(24))

        ### Define url handling
        def open_url(dialog, link, data=None):
            pass

        gtk.about_dialog_set_url_hook(open_url, None)

        self.menu = gui.get_status_icon_menu()
        
        self.window = gui.Window()

        self.connect('activate', self.on_activate)
        self.connect('popup-menu', self.on_popup)
        

        ### Run rss watcher code in a separate thread
        ### to not freeze the gui
        self.rss_thread = NotifyThread(debug)
        self.rss_thread.start()

    def on_popup(self, status_icon, mbutton, time, data=None):

        self.menu.popup(None, None,
                gtk.status_icon_position_menu, 3, time, self)


    def on_activate(self, status_icon, data=None):
        self.window.toggle_visible()
    
    def quit(self):
        gtk.main_quit()
        sys.exit(0)


def main(debug):

    gtk.gdk.threads_init()
    
    rssn = RssNotify(debug) 
    
    gtk.quit_add(0, rssn.rss_thread.quit)

    gtk.main()
    sys.exit(0)

